import { Component, OnInit } from '@angular/core';
import { TagEnum } from '../../enum/tag.enum';
import { Pagination } from '../../models/pagination.model';
import { Post } from '../../models/post.model';
import { Tag } from '../../models/tag.model';
import { PostWithMeta } from '../../models/postWithMeta.model';
import { PostService } from '../../services/post.service';
import { ActivatedRoute, Router } from '@angular/router';
import * as _ from 'lodash';
import { parseSlugToTag } from '../utils/NewsUtils';

@Component({
  selector: 'app-post-list',
  templateUrl: './post-list.component.html',
  styleUrls: ['./post-list.component.scss'],
})
export class PostListComponent implements OnInit {
  public selectedMainTagSlug = '';
  public selectedLocationTagSlug = [];
  public selectedPublicTagsSlug = [];
  public filters: Tag[];
  public allPosts: Post[] = [];
  public pagination: Pagination;
  public isLoading = false;
  public isPublishMode = false;
  public isLastPage: boolean;

  constructor(private postService: PostService, private route: ActivatedRoute, private router: Router) {}

  ngOnInit(): void {
    this.isLoading = true;
    // Init APP news list
    this.route.queryParams.subscribe((queryParams) => {
      this.isPublishMode = false;
      // If main tag is in route, set it
      if (queryParams.mainTag) {
        this.selectedMainTagSlug = queryParams.mainTag;
        this.selectedPublicTagsSlug = parseSlugToTag(queryParams.publicTags);
        this.selectedLocationTagSlug = parseSlugToTag(queryParams.locationTags);
        // Set filters for search and display
        this.filters = [
          new Tag({ slug: queryParams.mainTag }),
          ...this.selectedLocationTagSlug,
          ...this.selectedPublicTagsSlug,
        ];
        // Reset posts
        this.resetPosts();
        // Apply search
        this.getPosts(1, this.filters);
      } else {
        // Init default news list
        this.allPosts = [];
        this.filters = [];
        this.postService.getPosts(1).subscribe((news) => {
          this.fillArticles(news);
        });
      }
    });
  }

  /**
   * Fill articles list with headline handling
   * @param news {PostWithMeta}
   */
  public fillArticles(news: PostWithMeta): void {
    this.setNews(news);
    news.meta.pagination.pages > news.meta.pagination.page ? (this.isLastPage = false) : (this.isLastPage = true);
    const headLineTag = this.allPosts.filter((post: Post) =>
      post.tags.some((tag) => tag && tag.slug === TagEnum.aLaUne)
    );

    if (headLineTag.length) {
      const headIndex = this.allPosts.findIndex((post) => post.id === headLineTag[0].id);
      this.allPosts.splice(headIndex, 1);
    }

    this.allPosts = [...headLineTag, ..._.difference(this.allPosts, headLineTag)];
  }

  public getPosts(page: number, filters?: Tag[]): void {
    // Parse filter
    let parsedFilters = null;
    if (filters) {
      parsedFilters = filters.map((filter) => {
        return filter.slug;
      });

      if (parsedFilters.length <= 0) {
        parsedFilters = null;
      }
    }

    this.isLoading = true;
    this.postService.getPosts(page, parsedFilters).subscribe((news) => {
      this.fillArticles(news);
    });
  }

  public getDisplayedTag(): string {
    if (!this.isALaUneTag()) {
      return this.selectedMainTagSlug;
    }
    return 'autres actualités';
  }

  public isALaUneTag(): boolean {
    if (!this.filters || this.filters[0].slug === TagEnum.aLaUne) {
      return true;
    }
    return false;
  }

  public resetPosts(): void {
    this.allPosts = [];
  }

  // Transform excerpt post to have a custom author.
  private addAuthorToPost(post: Post): Post {
    post.author = post.excerpt;
    if (post.html) post.excerpt = post.html.replace(/<[^>]*>/g, '');
    return post;
  }

  // Load more news on scroll event.
  public loadMore(): void {
    if (this.pagination && this.pagination.page < this.pagination.pages) {
      this.isLoading = true;
      if (this.filters) {
        this.getPosts(this.pagination.next, this.filters);
      } else {
        this.postService.getPosts(this.pagination.next).subscribe((news) => {
          this.fillArticles(news);
        });
      }
    }
  }

  // Split news on two columns on desktop mode or one column in mobile mode.
  private setNews(news: PostWithMeta): void {
    this.pagination = news.meta.pagination;
    const newPosts = news.posts.map((val) => {
      val = this.addAuthorToPost(val);
      return val;
    });
    this.allPosts = [...this.allPosts, ...newPosts];
    this.isLoading = false;
  }

  public removeTag(tagToRemove: Tag): void {
    _.remove(this.selectedPublicTagsSlug, { slug: tagToRemove.slug });
    _.remove(this.selectedLocationTagSlug, { slug: tagToRemove.slug });
    this.router.navigate([], {
      relativeTo: this.route,
      queryParams: {
        mainTag: this.selectedMainTagSlug,
        publicTags: this.selectedPublicTagsSlug.map((tag) => tag.slug),
        locationTags: this.selectedLocationTagSlug.map((tag) => tag.slug),
      },
      queryParamsHandling: 'merge',
    });
  }

  public displayTags(): boolean {
    return this.selectedLocationTagSlug.length > 0 || this.selectedPublicTagsSlug.length > 0;
  }
}
